using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using UnityEngine.UI;
using UnityEngine.Events;


namespace Frief
{
    public class Doors : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Frief";
        public string pluginName = "Doors";
        public string pluginVersion = "1.1";
        public string pluginDate = "01/24" + "/2019";
        public string pluginDescription = @"This plugin controls the Doors of a Car as long as the are named Door1 ... Door2 ... ";
        #endregion

        #region Vars    



        public GameObject player;
        protected JSONStorableStringChooser uiFocusTarget;
        public GameObject Car;

        protected JSONStorableFloat ScrubA;
        protected JSONStorableFloat ScrubB;
        protected JSONStorableFloat ScrubC;
        protected JSONStorableFloat ScrubD;
        protected JSONStorableFloat ScrubSteering;


        private float previousValue1;
        private float previousValue2;
        private float previousValue3;
        private float previousValue4;
        private float previousFR;
        private float previousFL;
        private float previousStreeringValue;

        private float previousZValue;
        private float currentZValue;

        public Transform Cardoor1;
        public Transform Cardoor2;
        public Transform Cardoor3;
        public Transform Cardoor4;
        public Transform weelFR;
        public Transform weelFL;
        public Transform weelRR;
        public Transform weelRL;
        public Transform Steering;



        private static Atom currentAtom;
        private static string currentAtomName = "None";
        private static string tempAtomName = "None";

        #endregion

        public override void Init()
        {
            try
            {

                #region Left Side UI


                //**************  ScrubA
                ScrubA = new JSONStorableFloat("Door Back Left", 0.706f, 0.706f, 1.16f, false);
                RegisterFloat(ScrubA);
                CreateSlider(ScrubA, false);
                          

                //**************  ScrubB
                ScrubB = new JSONStorableFloat("Door Back right", 0f, 0f, 0.56f, false);
                RegisterFloat(ScrubB);
                CreateSlider(ScrubB, false);

                //**************  ScrubC
                ScrubC = new JSONStorableFloat("Door Front Left", 0f, 0f, 0.28f, false);
                RegisterFloat(ScrubC);
                CreateSlider(ScrubC, false);

                //**************  ScrubD
                ScrubD = new JSONStorableFloat("Door Front Right", 0f, 0f, 0.28f, false);
                RegisterFloat(ScrubD);
                CreateSlider(ScrubD, false);


                #endregion

                #region Right Side UI                
                //**************   Animator choices               
                // make atom selector


                List<string> targetChoices = new List<string>();
                foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
                {
                    currentAtom = SuperController.singleton.GetAtomByUid(atomUID);
                    
                        targetChoices.Add(atomUID);
                    
                }

                // want to always resync the atom choices on opening popup since atoms can be added/removed

                uiFocusTarget = new JSONStorableStringChooser("Choose Car Asset", targetChoices, "None", "Choose Car Asset");
                RegisterStringChooser(uiFocusTarget);
                UIDynamicPopup udp = CreatePopup(uiFocusTarget, true);


                //**************  ScrubD
                ScrubSteering = new JSONStorableFloat("Stearing", 0f, -0.13f, 0.13f, false);
                RegisterFloat(ScrubSteering);
                CreateSlider(ScrubSteering, false);

                #endregion


            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void Start()
        {
            

            previousValue1 = ScrubA.val;
            previousValue2 = ScrubB.val;
            previousValue3 = ScrubC.val;
            previousValue4 = ScrubD.val;
            currentAtomName = uiFocusTarget.val;
            

        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
           
            var Slider1 = ScrubA.val;
            var Slider2 = ScrubB.val;
            var Slider3 = ScrubC.val;
            var Slider4 = ScrubD.val;
            var Slider5 = ScrubSteering.val;
            currentAtomName = uiFocusTarget.val;
            


            // Checking uiFocusTarget  Dropdown has changed
            // #VALUE currentAtomName
            if (currentAtomName != "None")
            {

                // if not "None" check if it changed to prevouis Value
                //# VALUE tempAtomName
                if (tempAtomName != uiFocusTarget.val)
                {

                    // If changed store current selection in tempAtomName
                    // Start FUNCTION findCar();
                    findCar();
                    tempAtomName = uiFocusTarget.val;
                    previousValue1 = ScrubA.val;
                    previousValue2 = ScrubB.val;
                    previousValue3 = ScrubC.val;
                    previousValue4 = ScrubD.val;
                    previousStreeringValue=0f;

                }

                else
                {
                    tempAtomName = uiFocusTarget.val;
                    currentZValue = this.transform.position.x;
                    
                }

                if (currentZValue != previousZValue)
                {
                    var i = currentZValue - previousZValue;
                    SuperController.LogError("Change"+ i);
                    weelFR.Rotate(Vector3.right * i * Mathf.PI *-60);
                    weelFL.Rotate(Vector3.right * i * Mathf.PI * -60);
                    weelRR.Rotate(Vector3.right * i * Mathf.PI * -60);
                    weelRL.Rotate(Vector3.right * i * Mathf.PI * -60);
                    previousZValue = this.transform.position.x;
                }

                if (previousValue1 != Slider1)
                {
                    OnSliderChanged();
                }

                if (previousValue2 != Slider2)
                {
                    OnSliderChanged2();
                }

                if (previousValue3 != Slider3)
                {
                    OnSliderChanged3();
                }

                if (previousValue4 != Slider4)
                {
                    OnSliderChanged4();
                }

                if(previousStreeringValue!= Slider5)
                {
                    streeringChanged();
                }

            }

        }

        void streeringChanged()
        {
            if (currentAtomName == "None")
            {
                SuperController.LogError("No Car Asset is selected ");
            }
            else
            {

                var Slider5 = ScrubSteering.val;
                var atemp = Slider5 - previousStreeringValue;
                weelFR.transform.Rotate(Vector3.up * atemp * 360, Space.World);
                weelFL.transform.Rotate(Vector3.up * atemp * 360, Space.World);
                Steering.transform.Rotate(Vector3.up * atemp * 360);
                previousStreeringValue = ScrubSteering.val;

            }
        }

        void OnSliderChanged()
        {
            

            if (currentAtomName == "None")
            {
                SuperController.LogError("No Car Asset is selected ");
            }
            else
            {

                var Slider1 = ScrubA.val;
                var a = Slider1 - previousValue1;
               Cardoor1.transform.Rotate(Vector3.up * a * 360);
               previousValue1 = ScrubA.val;
            }

        }


        void OnSliderChanged2()
        {


            if (currentAtomName == "None")
            {
                SuperController.LogError("No Car Asset is selected ");
            }
            else
            {

                var Slider2 = ScrubB.val;
                var a = Slider2 - previousValue2;
                Cardoor2.transform.Rotate(Vector3.up * a * -360);
                previousValue2 = ScrubB.val;
            }

        }
        void OnSliderChanged3()
        {


            if (currentAtomName == "None")
            {
                SuperController.LogError("No Car Asset is selected ");
            }
            else
            {

                var Slider3 = ScrubC.val;
                var b = Slider3 - previousValue3;
                Cardoor3.transform.Rotate(Vector3.up * b * 360);
                previousValue3= ScrubC.val;
            }

        }
        void OnSliderChanged4()
        {


            if (currentAtomName == "None")
            {
                SuperController.LogError("No Car Asset is selected ");
            }
            else
            {

                var Slider4 = ScrubD.val;
                var c = Slider4 - previousValue4;
                Cardoor4.transform.Rotate(Vector3.up * c * -360);
                previousValue4 = ScrubD.val;
            }

        }

        public void findCar()
        {

            if (currentAtomName == "None")
            {
                SuperController.LogError("No Car Asset is selected ");
            }

            Car = GameObject.Find("CustomUnityAsset");

            var findDoor = "reParentObject/object/rescaleObject/Car/ContollbackLeft";
            var findDoor2 = "reParentObject/object/rescaleObject/Car/ContollbackRight";
            var findDoor3 = "reParentObject/object/rescaleObject/Car/DoorFrontLeft";
            var findDoor4 = "reParentObject/object/rescaleObject/Car/DoorFrontRight";

            var findweelFR = "reParentObject/object/rescaleObject/Car/Weels/Wheel_FR";
            var findweelFL = "reParentObject/object/rescaleObject/Car/Weels/Wheel_FL";
            var findweelRR = "reParentObject/object/rescaleObject/Car/Weels/Wheel_RR";
            var findweelRL = "reParentObject/object/rescaleObject/Car/Weels/Wheel_RL";

            var findSteeringweel = "reParentObject/object/rescaleObject/Car/Steering";

            Cardoor1 = containingAtom.transform.Find(findDoor);
            Cardoor2 = containingAtom.transform.Find(findDoor2);
            Cardoor3 = containingAtom.transform.Find(findDoor3);
            Cardoor4 = containingAtom.transform.Find(findDoor4);

            weelFR=containingAtom.transform.Find(findweelFR);
            weelFL=containingAtom.transform.Find(findweelFL); 
            weelRR = containingAtom.transform.Find(findweelRR);
            weelRL = containingAtom.transform.Find(findweelRL); 
            Steering = containingAtom.transform.Find(findSteeringweel);
            
            float yRotation = Cardoor1.transform.localRotation.y;
            float yRotation2 = Cardoor2.transform.localRotation.y;
            float yRotation3 = Cardoor3.transform.localRotation.y;
            float yRotation4 = Cardoor4.transform.localRotation.y;

            float yrotaionFR = weelFR.transform.localRotation.y;
            float yrotaionFl = weelFL.transform.localRotation.y;
            float yrotaionSteering = Steering.transform.localRotation.y;

            ScrubA.val = yRotation;
            ScrubB.val = yRotation2;
            ScrubC.val = yRotation3;
            ScrubD.val = yRotation4;

            previousFR= yrotaionFR;
            previousFL= yrotaionFl;
            previousStreeringValue= yrotaionFl;

            previousZValue = this.transform.position.x;

            if (Car != null)
            {
                


                if (Cardoor1 == null)
               {
                    SuperController.LogError("No child with the name " + findDoor + " attached to the player");
                }

                if (Cardoor2 == null)
                {
                    SuperController.LogError("No child with the name " + findDoor2 + " attached to the player");
                }

                if (Cardoor3 == null)
                {
                    SuperController.LogError("No child with the name " + findDoor3+ " attached to the player");
                }

                if (Cardoor4 == null)
                {
                    SuperController.LogError("No child with the name " + findDoor4 + " attached to the player");
                }

            }

        }

    }
}